Barrel = LibStub("AceAddon-3.0"):NewAddon("Barrel", "AceConsole-3.0", "AceEvent-3.0")
local ldb = LibStub("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale("Barrel")
local MinimapShapes = {
	-- quadrant booleans (same order as SetTexCoord)
	-- {upper-left, lower-left, upper-right, lower-right}
	-- true = rounded, false = squared
	["ROUND"] 			= {true, true, true, true},
	["SQUARE"] 			= {false, false, false, false},
	["CORNER-TOPLEFT"] 		= {true, false, false, false},
	["CORNER-TOPRIGHT"] 		= {false, false, true, false},
	["CORNER-BOTTOMLEFT"] 		= {false, true, false, false},
	["CORNER-BOTTOMRIGHT"]	 	= {false, false, false, true},
	["SIDE-LEFT"] 			= {true, true, false, false},
	["SIDE-RIGHT"] 			= {false, false, true, true},
	["SIDE-TOP"] 			= {true, false, true, false},
	["SIDE-BOTTOM"] 		= {false, true, false, true},
	["TRICORNER-TOPLEFT"] 		= {true, true, true, false},
	["TRICORNER-TOPRIGHT"] 		= {true, false, true, true},
	["TRICORNER-BOTTOMLEFT"] 	= {true, true, false, true},
	["TRICORNER-BOTTOMRIGHT"] 	= {false, true, true, true},
}


function Barrel:OnInitialize()

	-- Called when the addon is loaded
	self:Config()
	self:RegisterChatCommand("br", "ChatCommand")
	self:RegisterChatCommand("barrel", "ChatCommand")

end

function Barrel:OnEnable()
	for name, dataobj in ldb:DataObjectIterator() do
		local frame = "Barrel"..name
		self:CreateFrame(nil, name, dataobj)
		self:UpdatePosition(name, dataobj)
		self:CreateMenu(nil, name, dataobj)
	end
	ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged__icon", "AttributeChanged_icon")
	ldb.RegisterCallback(self, "LibDataBroker_DataObjectCreated", "DataObjectCreated")
end

function Barrel:DataObjectCreated(event, name, dataobj)
	local event, name, dataobj = event, name, dataobj
	self:CreateFrame(nil, name, dataobj)
	self:UpdatePosition(name)
	self:CreateMenu(nil, name, dataobj)
end

function Barrel:ChatCommand(input)

	local input = input

	if not input or input:trim() == "" then
	  LibStub("AceConfigDialog-3.0"):Open("Barrel")
	else
	  LibStub("AceConfigCmd-3.0").HandleCommand(Barrel, "br", "Barrel", input)
	end

end

function Barrel:CreateFrame(event, name, dataobj)
	local event, name, dataobj = event, name, dataobj
	local frame = "Barrel"..name
	if self.db.profile[frame] == nil then
		self.db.profile[frame] = {}
	end
	self.db.profile[frame].Minimap = CreateFrame('Button', frame, Minimap)
	local minimap = self.db.profile[frame].Minimap
	minimap:SetFrameStrata('MEDIUM')
	minimap:SetWidth(31); minimap:SetHeight(31)
	minimap:SetFrameLevel(8)
	minimap:RegisterForClicks('anyUp')
	minimap:RegisterForDrag('LeftButton')
	minimap:SetHighlightTexture('Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight')

	local overlay = minimap:CreateTexture(nil, 'OVERLAY')
	overlay:SetWidth(53); overlay:SetHeight(53)
	overlay:SetTexture('Interface\\Minimap\\MiniMap-TrackingBorder')
	overlay:SetPoint('TOPLEFT')

	local icon = minimap:CreateTexture(nil, 'BACKGROUND')
	icon:SetWidth(20); icon:SetHeight(20)
	icon:SetTexture(dataobj.icon)
	icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	icon:SetPoint('TOPLEFT', 7, -5)
	self.db.profile[frame].icon = icon

	local elap = 0
	local update = 15
	minimap:SetScript('OnEnter', function() self:OnEnter(dataobj) end)
	minimap:SetScript('OnLeave', function() self:OnLeave(dataobj) end)
	minimap:SetScript('OnClick', function(_, button) self:OnClick(dataobj, button) end)
	minimap:SetScript('OnDragStart', function() self:OnDragStart(name) end)
	minimap:SetScript('OnDragStop', function() self:OnDragStop(name) end)
	minimap:SetScript('OnMouseDown', function() self:OnMouseDown(name) end)
	minimap:SetScript('OnMouseUp', function() self:OnMouseUp(name) end)
	minimap:SetScript('OnUpdate', 
		function(_, elapsed) 
			local elapsed = elapsed
			elap = elap + elapsed
			if elap <= update then 
				self:UpdatePosition(name)
				minimap:SetScript('OnUpdate', nil)
			end
		end
	)
end

function Barrel:OnClick(dataobj, button)
	local dataobj, button = dataobj, button
	
	dataobj.OnClick(this, button)
	
	self:OnEnter(dataobj)
end

function Barrel:OnMouseDown(name)
	local name = name
	local frame = "Barrel"..name
	self.db.profile[frame].icon:SetTexCoord(0, 1, 0, 1)
end

function Barrel:OnMouseUp(name)
	local name = name
	local frame = "Barrel"..name
	self.db.profile[frame].icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
end

function Barrel:OnEnter(dataobj)
	local dataobj = dataobj
	if dataobj.OnEnter then
			dataobj.OnEnter(this)
	end
	if dataobj.OnTooltipShow then
		dataobj.OnTooltipShow(GameTooltip)
		GameTooltip:Show()
	elseif dataobj.tooltip then
		dataobj.tooltip:Show()
		if dataobj.tooltiptext then
			dataobj.tooltip:SetText(dataobj.tooltiptext)
		end
	elseif dataobj.tooltiptext then
		GameTooltip:SetText(dataobj.tooltiptext)
		GameTooltip:Show()
	end
end

function Barrel:OnLeave(dataobj)
	if dataobj.OnLeave then
		dataobj.OnLeave()
	else
		GameTooltip:Hide()
	end
end

function Barrel:OnDragStart(name)
	local name = name
	local frame = "Barrel"..name
	local minimap = self.db.profile[frame].Minimap
	local icon = self.db.profile[frame].icon
	self.dragging = true
	minimap:LockHighlight()
	icon:SetTexCoord(0, 1, 0, 1)
	minimap:SetScript('OnUpdate', function() self:OnUpdate(name) end)
	GameTooltip:Hide()
end

function Barrel:OnDragStop(name)
	local name = name
	local frame = "Barrel"..name
	local minimap = self.db.profile[frame].Minimap
	local icon = self.db.profile[frame].icon
	self.dragging = nil
	minimap:SetScript('OnUpdate', nil)
	icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	minimap:UnlockHighlight()
end

function Barrel:OnUpdate(name)
	local name = name
	local mx, my = Minimap:GetCenter()
	local px, py = GetCursorPosition()
	local scale = Minimap:GetEffectiveScale()

	px, py = px / scale, py / scale

	self:SetMinimapButtonPosition(name, math.deg(math.atan2(py - my, px - mx)) % 360)
	self:UpdatePosition(name)
end

function Barrel:UpdatePosition(name)
	local name = name
	local frame = "Barrel"..name
	
	local minimap = self.db.profile[frame].Minimap
	local radius = self.db.profile[frame].radius or 80
	local rounding = self.db.profile[frame].rounding or 10
	local position = self:GetMinimapButtonPosition(name) or random(0, 360)
	self:SetMinimapButtonPosition(name, position)
	self.db.profile[frame].radius = radius
	angle = math.rad(position)
	local x = math.cos(angle)
	local y = math.sin(angle)
	local q = 1;
	if x < 0 then
		q = q + 1;
	end
	if y > 0 then
		q = q + 2;
	end
	local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
	local quadTable = MinimapShapes[minimapShape];
	if quadTable[q] then
		x = x*radius;
		y = y*radius;
	else
		local diagRadius = math.sqrt(2*(radius)^2)-rounding
		x = math.max(-radius, math.min(x*diagRadius, radius))
		y = math.max(-radius, math.min(y*diagRadius, radius))
	end
	minimap:SetPoint("CENTER", Minimap, "CENTER", x, y)
	
	if self.db.profile[frame].Hidden then
		self:Hide(name)
	end
end

function Barrel:SetMinimapButtonPosition(name, angle)
	local name = name
	frame = "Barrel"..name
	self.db.profile[frame].minimapPos = angle
end

function Barrel:GetMinimapButtonPosition(name)
	local name = name
	frame = "Barrel"..name
	return self.db.profile[frame].minimapPos
end

function Barrel:Show(name)
	local frame = "Barrel"..name
	self.db.profile[frame].Minimap:Show()
end

function Barrel:Hide(name)
	local frame = "Barrel"..name
	self.db.profile[frame].Minimap:Hide()
end

function Barrel:IsLocked()
	return --self.locked
end

function Barrel:AttributeChanged_icon(event, name, key, value, dataobj)
	local event, name, key, value, dataobj = event, name, key, value, dataobj
	--name = string.gsub(name, "FuBar(%s)\-(%s)", "")
	local frame = "Barrel"..name
	self.db.profile[frame].icon:SetTexture(dataobj.icon)
end


-- Copyright 2008 Copystring"